<?php

class I24H
{
    public static $SERVER = 'api.d.iwork.ru';
}

const I24H_ID_URL = 'http://%s/v1/access?vendor_id=2&device_id=%s&stream=orig&_format=json';

const I24H_LOGIN_URL =
    'http://%s/v1/login?vendor_id=2&phone=%s&password=%s&device_id=%s&access-token=%s&_format=json';
    
const I24H_LOGOUT_URL =
    'http://%s/v1/logout?access-token=%s&_format=json';
    
const I24H_GET_URL_URL =
    'http://ts://%s/dune/plugini.php?action=get_url&login=%s&pass=%s';
    
const I24H_SEND_PWD_URL =
    'http://%s/v1/reset?vendor_id=2&phone=%s&device_id=%s&_format=json';

const I24H_SERVICES_URL =
    'http://%s/v1/tariffs/search?access-token=%s&expand=channels&sort=priorety&_format=json';

const I24H_SERVICES_AV_URL =
    'http://%s/v1/payment/available?access-token=%s&_format=json';

const I24H_PAYSYSTEMS_URL =
    'http://%s/v1/paysystem/search?access-token=%s&active=1&_format=json';

const I24H_INV_URL =
    'http://%s/v1/profile/invoice?access-token=%s&_format=json';

const I24H_BALANCE_URL =
    'http://%s/v1/payment/cont?access-token=%s&_format=json';

const I24H_ADD_SERVICE =
    'http://%s/v1/payment/create?access-token=%s&trf=%s&_format=json';

const I24H_DEL_SERVICE =
    'http://%s/v1/payment/delete?access-token=%s&trf=%s&_format=json';
    
class I24hSession
{
    private $http_opts = null;
    //private $logged_in = false;
    private $sid_name = null;
    private $sid = null;
    private $sid_uid = null;
    private $token = null;
    private $mbr_id = null;

    private $login_incorrect = false;

    private $account = null;
    private $services = null;
    private $settings = null;

    private $channel_list = null;
    private $unset_login_listener = null;
    private $service_list = null;
    private $paysystems_list = null;

    public function __construct($unset_login_listener)
    {
        $this->http_opts = array (
            CURLOPT_FOLLOWLOCATION  => true,
            CURLOPT_MAXREDIRS       => 10,
// debug            CURLOPT_PROXY           => '192.168.1.9:3128',
            CURLOPT_COOKIEFILE      => '/tmp/24h2.cookies',
            CURLOPT_COOKIEJAR       => '/tmp/24h2.cookies'
        );

        $this->unset_login_listener = $unset_login_listener;
    }
    
    public function get_http_opts()
    { return $this->http_opts; }

    public function get_sid()
    { return $this->sid; }

    public function get_sid_uid()
    { return $this->sid_uid; }
    
    public function get_token()
    { return $this->token; }

    public function get_channel_list()
    { return $this->channel_list; }

    public function get_account()
    { return $this->account; }

    public function get_settings()
    { return $this->settings; }
    
    public function is_login_incorrect()
    { return $this->login_incorrect; }

    public function is_logged_in()
    { return ($this->mbr_id>0); }

    public function unset_login()
    {
        $this->logged_in = false;
        $this->sid_name = null;
        $this->sid = null;

        $this->account = null;
        $this->services = null;

        $this->settings = null;

        $this->channel_list = null;
    }

    public function logout()
    {
    	if (isset($this->token)) {
    	    try
    	    {
    	    	$doc = $this->api_call(sprintf(I24H_LOGOUT_URL, I24H::$SERVER, $this->token));
    	    }
    	    catch (Exception $e)
    	    {
    	    	throw $this->dune_api_exception($e,
    	    	    'Stream url request failed.',
    	    	    true);
    	    }
    	    unset($this->token);
    	}
    	
        $this->unset_login_listener->unset_login();
    }

    ///////////////////////////////////////////////////////////////////////

    private function fetch_document($url)
    {
        $doc = HD::http_get_document($url, $this->http_opts);

        // assert($doc != null).

        return $doc;
    }

    private function fetch_post_document($url, $data)
    {
        $doc = HD::http_post_document($url, $data, $this->http_opts);

        // assert($doc != null).

        return $doc;
    }

    public function api_call($url, $silent=true, $data=null)
    {
        for (;;)
        {
            hd_print("api_call: $url");
            if (isset($data)) {
            	$str = $this->fetch_post_document($url, $data);
            } else {
            	$str = $this->fetch_document($url);
            }

            if (!$silent)
                echo("Document:\n$str\n");

            $reply = json_decode($str);
            if ($reply === null)
                throw new Exception('Invalid data received from server');

            /*if ($reply->res != 'ok')
            {
                $ecode = 20; //intval($reply->error->code);

                if ($ecode === ECODE_QUERY_LIMIT_EXCEEDED)
                {
                    hd_print('API: query limit exceeded');
                    continue;
                }

                hd_print("API error: URL '$url' " .
                    "returned error with code=$ecode, msg=" .
                    $reply->message . '.');

                throw new I24hException(
                    $reply->message, $ecode);
            }*/

            break;
        }

        return $reply;
    }

    public function dune_api_exception($e, $def_caption, $is_playback)
    {
        $is_i24h_error = ($e instanceof I24hException);
        if (!$is_i24h_error)
        {
            hd_print('General exception: ' . $e->getMessage());
            $title = !$is_playback ? 'Error' :
                'Failed to connect to Internet.';
            $text_lines = $is_playback ? array() :
                array(
                 'Failed to connect to Internet.',
                 'Please check Internet connection.'
                );
            return new DuneException(
                $def_caption, -1,
                ActionFactory::show_error(false, $title, $text_lines));
        }
        hd_print('24h.TV API exception: ' .
            'code=' . $e->getCode() . ', message=' . $e->getMessage());

        $ecode = $e->getCode();
        $fatal = 
            $ecode == 11 || $ecode == 12 || $ecode == 13;

        if ($fatal)
            $this->logout();

        if ($ecode == 2)
        {
            $this->login_incorrect = true;

            $title = !$is_playback ? 'Ошибка' :
                'Неправильные данные подписки, проверьте номер и пароль.';
            $text_lines = $is_playback ? array() :
                array('Неправильные данные подписки, проверьте номер и пароль.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 4)
        {
            $title = !$is_playback ? 'Ошибка' :
                'Подписка истекла. Зайдите на 24h.tv и пополните счет для ее продления.';
            $text_lines = $is_playback ? array() :
                array('Подписка истекла. Зайдите на 24h.tv и пополните счет для ее продления.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 5)
        {
            $title = !$is_playback ? 'Ошибка' :
                'Неправильный телефонный номер. Используейте формат 89XXXXXXXXX (11 цифр).';
            $text_lines = $is_playback ? array() :
                array('Неправильный телефонный номер. Используейте формат 89XXXXXXXXX (11 цифр).');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }
        
        if ($ecode == 6)
        {
            $title = !$is_playback ? 'Ошибка' :
                'Слишком много запросов. Попробуйте попозже.';
            $text_lines = $is_playback ? array() :
                array('Слишком много запросов. Попробуйте попозже.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 20)
        {
            $title = !$is_playback ? 'Ошибка' :
                $e->getMessage();
            $text_lines = $is_playback ? array() :
                array($e->getMessage());
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

 /*        if ($ecode == 3)
        {
            $title = !$is_playback ? 'Error' :
                'Access denied for 10 minutes.';
            $text_lines = $is_playback ? array() :
                array('Access denied for 10 minutes.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 12)
        {
            $title = !$is_playback ? 'Error' :
                'Access denied. Probably subscription is used elsewhere.';
            $text_lines = $is_playback ? array() :
                array('Access denied.',
                    'Probably subscription is used elsewhere.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        if ($ecode == 11)
        {
            $title = !$is_playback ? 'Error' :
                'Subscription is used elsewhere.';
            $text_lines = $is_playback ? array() :
                array('Subscription is used elsewhere.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }*/

        if ($ecode == 19)
        {
            $title = !$is_playback ? 'Error' :
                'Current code is wrong.';
            $text_lines = $is_playback ? array() :
                array('Current code is wrong.');
            return new DuneException(
                $def_caption, $ecode,
                ActionFactory::show_error($fatal, $title, $text_lines));
        }

        $title = !$is_playback ? 'Ошибка' : $e->getMessage();
        $text_lines = $is_playback ? array() :
            array(rtrim($e->getMessage()),
                'Error code: ' . $ecode);
        return new DuneException(
            $def_caption, $ecode,
            ActionFactory::show_error($fatal, $title, $text_lines));
    }
    
    public function get_device_id() {
    	$h = fopen('/fconfig/dune_license.dlf', "r");
    	$l = fgets($h);
    	fclose($h);
    	
    	//hd_print($l);
    	preg_match('/\d+ \d+ (\S+) /', $l, $matches);
    	return preg_replace('/-/','',$matches[1]);
    }
    
    public function verify_device() {
    	$device_id = $this->get_device_id();
    	
	hd_print("Trying to verify, device id ".$device_id);
    	
    	if ($device_id) { 
    	    
    	    try
    	    {
    	    	$doc = $this->api_call(
    	    	    sprintf(
    	    	    	I24H_ID_URL, I24H::$SERVER,
    	    	    	$device_id),
    	    	    false);
    	    }
    	    catch (Exception $e)
    	    {
    	    	throw $this->dune_api_exception($e, 'Verify failed.', false);
    	    }
    	    $this->token = $doc->token;
    	    $this->mbr_id = $doc->mbr_id;
    	    $this->sid = $doc->stream->sid;
    	    $this->sid_name = 'sid';
    	    $this->sid_uid = $doc->stream->uid;
    	    
    	    hd_print("Verify ok, token=$this->token, mbr_id=$this->mbr_id, sid=$this->sid, uid=$this->sid_uid");
    	}
    }

    public function api_get_stream_url($playback_url)
    {
        $this->check_logged_in();

        if (substr($playback_url, 0, 12) === 'http://ts://')
            $playback_url = 'http://' . substr($playback_url, 12);
        else if (substr($playback_url, 0, 13) === 'http://mp4://')
            $playback_url = 'http://' . substr($playback_url, 13);

        try
        {
            $doc = $this->api_call($playback_url);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        if (!isset($doc->url))
            throw new Exception('Get_url failed.');

        $len = strlen($doc->url);
        $pos = $len;
        for ($i = 10; $i < $len; $i++)
        {
            if ($doc->url[$i] == '"' || $doc->url[$i] == ' ')
            {
                $pos = $i;
                break;
            }
        }

        $url = substr($doc->url, 0, $pos);

        if (substr($url, 0, 10) === 'http/ts://')
            return 'http://' . substr($url, 10);

        return $url;
    }


    private function api_login($user_name, $password)
    {
    	$device_id = $this->get_device_id();
    	
	hd_print("Trying to login, device id ".$device_id);
        try
        {
            
            $doc = $this->api_call(
                sprintf(
                    I24H_LOGIN_URL, I24H::$SERVER,
                    preg_replace('/^8/', '7', $user_name), $password, $device_id, $this->token),
                false);
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e, 'Login failed.', false);
        }
        if ($doc->res == 'err') {
            throw $this->dune_api_exception(new I24hException($doc->message, 12), $doc_message, false);
//            throw $this->dune_api_exception(new Exception($doc->message), $doc->message, false);
        }

        
        
        /*$this->logged_in = true;
        $this->sid = $doc->sid;
        $this->sid_name = $doc->sid_name;
        $this->login_incorrect = false;
        $this->account = $doc->account;
        $this->services = $doc->services;
        $this->settings = $doc->settings;*/
    }

    public function try_login(&$plugin_cookies)
    {
        if (!isset($plugin_cookies->user_name))
            throw new Exception('User name is not set');

        try
        {
            $this->verify_device();
            $this->api_login(
                $plugin_cookies->user_name,
                $plugin_cookies->password);

           $this->verify_device();
           unset($this->service_list);
           
           #$this->api_channel_list();
        }
        catch (Exception $e)
        {
            $this->logout();
            throw $e;
        }
    }

    public function ensure_logged_in(&$plugin_cookies)
    {
        if ($this->mbr_id)
            return;

        $this->verify_device();
        
        if (!$this->mbr_id) {
            throw $this->dune_api_exception(new I24hException('Вы не вошли в систему. Зайдите в настройки и укажите свой пароль.', 12), 12, false);
            //$this->try_login($plugin_cookies);
        }
    }

    public function check_logged_in()
    {
        if (!$this->mbr_id)
            throw new I24hException('Вы не вошли в систему. Зайдите в настройки и укажите свой пароль.', 12);
    }

    
    private function do_get_edit_subscription_defs(
        &$plugin_cookies, $handler, $add_params)
    {
        $defs = array();

        $user_name = isset($plugin_cookies->user_name) ?
            $plugin_cookies->user_name : '';
        $password = isset($plugin_cookies->password) ?
            $plugin_cookies->password : '';

        ControlFactory::add_label($defs,
            null, 'Смотри 150 каналов с помощью 24h.tv! Удобный выбор из 30000',
            array('indent' => 0));
        ControlFactory::add_vgap($defs, -15);
        ControlFactory::add_label($defs,
            null, 'передач и фильмов в архиве и Amediateka.',
            array('indent' => 0));
        ControlFactory::add_vgap($defs, 20);

        ControlFactory::add_text_field($defs,
            $handler, $add_params,
            'user_name', 'Телефон:',
            $user_name, true, false, false, false, 500);

        ControlFactory::add_text_field($defs,
            $handler, $add_params,
            'password', 'Пароль:', $password, 
            true, true, false, false, 500);

        ControlFactory::add_vgap($defs, 20);

        ControlFactory::add_button($defs,
            $handler, $add_params,
            'apply_subscription', null, 'Войти', 300);

        ControlFactory::add_vgap($defs, 20);
        ControlFactory::add_label($defs,
            null, 'Для получения бесплатного доступа к основным каналам введите',
            array('indent' => 0));
        ControlFactory::add_vgap($defs, -15);
        ControlFactory::add_label($defs,
            null, 'номер мобильного и нажмите кнопку для получения пароля по SMS:',
            array('indent' => 0));

        ControlFactory::add_vgap($defs, -10);
        ControlFactory::add_button($defs,
            $handler, $add_params,
            'get_password', null, 'Получить пароль', 500);
        
        
        ControlFactory::add_vgap($defs, 25);

        ControlFactory::add_close_dialog_button($defs,
            'Отмена', 300);

        return $defs;
    }
    
    public function do_get_edit_subscription_action(
        &$plugin_cookies, $handler, $add_params = null)
    {
        return ActionFactory::show_dialog(
            'Подписка на 24h.tv',
            $this->do_get_edit_subscription_defs(
                $plugin_cookies, $handler, $add_params),
            true);
    }

    public function apply_subscription(&$plugin_cookies, &$user_input)
    {
        if (!isset($user_input->control_id) ||
            $user_input->control_id != 'apply_subscription')
        {
            return false;
        }

        if ($user_input->user_name === '')
        {
            return array(
                'need_close_dialog' => false,
                'action' =>
                    ActionFactory::show_error(false,
                        'Error',
                        array('Subscription should be non-empty.')));
        }

        $plugin_cookies->user_name = $user_input->user_name;
        $plugin_cookies->password = $user_input->password;

        $this->logout();

        $post_action = null;
            hd_print("Trying to login");
        try
        {
            $this->try_login($plugin_cookies);
        }
        catch (DuneException $e)
        {
            $post_action = $e->get_error_action();

            $plugin_cookies->password = '';
        }

        return array(
            'need_close_dialog' => true,
            'action' => $post_action);
    }
    
    public function get_password(&$plugin_cookies, &$user_input)
    {
        if (!isset($user_input->control_id) ||
            $user_input->control_id != 'get_password')
        {
            return false;
        }

        if ($user_input->user_name === '')
        {
            return array(
                'need_close_dialog' => false,
                'action' =>
                    ActionFactory::show_error(false,
                        'Запрос пароля',
                        array('Введите номер телефона для отправки пароля.')));
        }

        try
        {
            $doc = $this->api_call(sprintf(I24H_SEND_PWD_URL, I24H::$SERVER, preg_replace('/^8/', '7', $user_input->user_name), $this->get_device_id()));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        return array(
                'need_close_dialog' => false,
                'action' =>
                ActionFactory::show_error(false,
                        'Запрос пароля',
                        array('На указанный номер отправлено SMS с паролем.',
                        	'Дождитесь сообщение, введите пароль и нажмите "Войти". '.$doc->message)));
        
        
        

    }

    private function retrieveServiceList()
    {
        try
        {
            $doc = $this->api_call(sprintf(I24H_SERVICES_URL, I24H::$SERVER, $this->token));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Stream url request failed.',
                true);
        }

        return $doc;
    }

    private function retrieveInvoicesList()
    {
        try
        {
            $doc = $this->api_call(sprintf(I24H_SERVICES_AV_URL, I24H::$SERVER, $this->token));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Invoices url request failed.',
                true);
        }

        return $doc;
    }

    private function retrievePaysystemsList()
    {
        try
        {
            $doc = $this->api_call(sprintf(I24H_PAYSYSTEMS_URL, I24H::$SERVER, $this->token));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Paysystems url request failed.',
                true);
        }

        return $doc;
    }

    private function updateInvoices()    
    {
        if (is_null($this->service_list))
            $this->loadServiceList();
        
        $invoices = $this->retrieveInvoicesList();
        $inv_arr = array();
        
        foreach ($invoices as $inv)
        {
            $inv_arr[$inv->id] = $inv;
        }
        
        foreach ($this->service_list as $srv)
        {
            if (isset($inv_arr[$srv->id])) {
                $srv->isSubscribed = 1;
                $srv->finished = $inv_arr[$srv->id]->finished;
            } else { 
            	$srv->isSubscribed = 0;
            }
        }
        
    }

    /*private function doAddServiceToSubscription($service_id)
    {
        $xml = $this->apiCall('addServiceToSubscription',
            array('serviceId' => $service_id));

        return $this->parseServiceList($xml);
    }

    private function doRemoveServiceFromSubscription($service_id)
    {
        $xml = $this->apiCall('removeServiceFromSubscription',
            array('serviceId' => $service_id));

        return $this->parseServiceList($xml);
    }*/
    
    private function loadServiceList()
    {
        $this->service_list = $this->retrieveServiceList();

        if (is_null($this->service_list))
            throw new Exception("Can not retrieve service list");
    }
    
    private function loadPaysystemsList()
    {
        $this->paysystems_list = $this->retrievePaysystemsList();
        
        if (is_null($this->paysystems_list))
            throw new Exception("Can not retrieve service list");
        
    }
    
    public function getServiceList()
    {
        if (is_null($this->service_list))
            $this->loadServiceList();
        
        $this->updateInvoices();

        return $this->service_list;
    }

    public function getPaysystemsList()
    {
        if (is_null($this->paysystems_list))
            $this->loadPaysystemsList();
        
        return $this->paysystems_list;
    }

    public function getServiceById($service_id)
    {
        if (is_null($this->service_list))
            $this->loadServiceList();

        $this->updateInvoices();

        foreach ($this->service_list as $srv)
        {
            if ($srv->id == $service_id)
                return $srv;
        }

        return null;
    }
    
    private function retrieveBalance()
    {
        try {
            $doc = $this->api_call(sprintf(I24H_BALANCE_URL, I24H::$SERVER, $this->token));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'Balance url request failed.',
                true);
        }

        return $doc->sum;   
    }
    
    public function getBalance()
    {
        $balance = $this->retrieveBalance();

        if (is_null($balance))
            throw new Exception("Can not retrieve balance.");

        return $balance;
    }
    
    public function addServiceToSubscription($service_id)
    {
        try {
            $doc = $this->api_call(sprintf(I24H_ADD_SERVICE, I24H::$SERVER, $this->token, $service_id));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'request add service failed.',
                true);
        }

        unset($this->service_list);
        return $doc;


    }

    public function removeServiceFromSubscription($service_id)
    {
        try {
            $doc = $this->api_call(sprintf(I24H_DEL_SERVICE, I24H::$SERVER, $this->token, $service_id));
        }
        catch (Exception $e)
        {
            throw $this->dune_api_exception($e,
                'request add service failed.',
                true);
        }

        unset($this->service_list);
        return $doc;
    }

    
}
