<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_controls_screen.php';

///////////////////////////////////////////////////////////////////////////

class I24hSetupScreen extends AbstractControlsScreen
{
    const ID = 'setup';
    private $session;

    ///////////////////////////////////////////////////////////////////////

    public function __construct($session)
    {
        parent::__construct(self::ID);
        $this->session = $session;
    }

    public function do_get_control_defs(&$plugin_cookies)
    {
        $defs = array();

        $user_name = isset($plugin_cookies->user_name) ?
            $plugin_cookies->user_name : '';

        $logged_in = $this->session->is_logged_in();

        $show_tv = isset($plugin_cookies->show_tv) ?
            $plugin_cookies->show_tv : 'yes';
        $show_vod = isset($plugin_cookies->show_vod) ?
            $plugin_cookies->show_vod : 'yes';

           
        $show_ops = array();
        $show_ops['yes'] = 'Yes';
        $show_ops['no'] = 'No';

        $this->add_combobox($defs,
            'show_tv', 'Показывать иконку в разделе ТВ:',
            $show_tv, $show_ops, 0, true);

        /*$this->add_combobox($defs,
            'show_vod', 'Show VOD in main screen:',
            $show_vod, $show_ops, 0, true);*/

        $this->add_button($defs, 'edit_subscription', null,
            'Подписка...', 0);

        return $defs;
    }

    public function get_control_defs(MediaURL $media_url, &$plugin_cookies)
    {
        return $this->do_get_control_defs($plugin_cookies);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('Setup: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $need_close_dialog = false;
        $need_reset_controls = false;
        $post_action = null;
        if ($user_input->action_type === 'apply')
        {
            $control_id = $user_input->control_id;
            if ($control_id === 'edit_subscription')
            {
                return $this->session->do_get_edit_subscription_action(
                    $plugin_cookies, $this);
            }
            else if ($control_id === 'apply_subscription')
            {
                if ($user_input->user_name === '')
                {
                    return ActionFactory::show_error(false,
                        'Error',
                        array('Subscription should be non-empty.'));
                }

                $plugin_cookies->user_name = $user_input->user_name;
                $plugin_cookies->password = $user_input->password;

                $this->session->logout();

                try
                {
                    $this->session->try_login($plugin_cookies);
                }
                catch (DuneException $e)
                {
                    $post_action = $e->get_error_action();
                    $plugin_cookies->password = '';
                }

                $need_close_dialog = true;
                $need_reset_controls = true;
            }
            else if ($control_id === 'get_password')
            {
                try
                {
            	    $res = $this->session->get_password($plugin_cookies, $user_input);
                }
                catch (DuneException $e)
                {
                    return $e->get_error_action();
                }
                $need_close_dialog = $res['need_close_dialog'];
                $need_reset_controls = $res['need_close_dialog'];
                return $res['action'];
            	    
            }
        } else if ($user_input->action_type === 'confirm')
        {
            $control_id = $user_input->control_id;
            $new_value = $user_input->{$control_id};
            hd_print("Setup: changing $control_id value to $new_value");

            if ($control_id === 'show_tv')
                $plugin_cookies->show_tv = $new_value;
            else if ($control_id === 'show_vod')
                $plugin_cookies->show_vod = $new_value;
            $need_reset_controls = true;
        }

        if ($need_reset_controls)
        {
            $defs = $this->do_get_control_defs($plugin_cookies);

            $reset_controls_action = ActionFactory::reset_controls(
                $defs, $post_action);

            if ($need_close_dialog)
            {
                return ActionFactory::close_dialog_and_run(
                    $reset_controls_action);
            }

            return $reset_controls_action;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
?>
