<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';

require_once 'i24h_channel.php';

///////////////////////////////////////////////////////////////////////////

class I24hTv extends AbstractTv
{
    private $session;

    public function __construct($session)
    {
        $this->session = $session;

        parent::__construct(
            AbstractTv::MODE_CHANNELS_N_TO_M,
            I24hConfig::TV_FAVORITES_SUPPORTED,
            true);
    }

    public function get_fav_icon_url()
    {
        return I24hConfig::FAV_CHANNEL_GROUP_ICON_PATH;
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    protected function load_channels(&$plugin_cookies)
    {
        $this->session->check_logged_in();
        
        $this->channels = new HashedArray();
        $this->groups = new HashedArray();

        if ($this->is_favorites_supported())
        {
            $this->groups->put(
                new FavoritesGroup(
                    $this,
                    '__favorites',
                    I24hConfig::FAV_CHANNEL_GROUP_CAPTION,
                    I24hConfig::FAV_CHANNEL_GROUP_ICON_PATH));
        }

        try
        {
            $doc = HD::http_get_document(
            	sprintf(I24hConfig::GROUP_LIST_URL, $this->session->get_token()),
            	$this->session->get_http_opts());
        }
        catch (Exception $e)
        {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }
     
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'response')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'response'");
            throw new Exception('Invalid XML document');
        }

        $first = 0;
        foreach ($xml->children() as $xml_tv_category)
        {
            if ($xml_tv_category->getName() !== 'item')
            {
                hd_print("Error: unexpected group node '" . $xml_tv_category->getName() .
                    "'. Expected: 'item'");
                throw new Exception('Invalid XML document');
            }

            $this->groups->put(
                new DefaultGroup(
                    strval($xml_tv_category->id),
                    strval($xml_tv_category->name),
                    strval($xml_tv_category->dune->url)));
            
            if (!$first) {
            	$first = 1;
            	$this->groups->put(
            	    new AllChannelsGroup(
            	    	$this,
            	    	I24hConfig::ALL_CHANNEL_GROUP_CAPTION,
            	    	I24hConfig::ALL_CHANNEL_GROUP_ICON_PATH));
            }
        }
        
        $active_channels = array();
        
        try 
        {
            $doc = HD::http_get_document(
            	sprintf(I24hConfig::CHANNEL_ACTIVE_URL, $this->session->get_token()),
            	$this->session->get_http_opts());
        }
        catch (Exception $e)
        {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }
        
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'response')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'response'");
            throw new Exception('Invalid XML document');
        }


        foreach ($xml->children() as $xml_tv_channel)
        {
            if ($xml_tv_channel->getName() !== 'item')
            {
                hd_print("Error: unexpected node '" . $xml_tv_channel->getName() .
                    "'. Expected: 'item'");
                throw new Exception('Invalid XML document');
            }
            
            $active_channels[strval($xml_tv_channel->id)] = 1;
        }        

        
        $active_list = array();
        $passive_list = array();
        
        try
        {
            $doc = HD::http_get_document(
            	sprintf(I24hConfig::CHANNEL_LIST_URL, $this->session->get_token()),
            	$this->session->get_http_opts());
        }
        catch (Exception $e)
        {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }
     
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'response')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'response'");
            throw new Exception('Invalid XML document');
        }


        foreach ($xml->children() as $xml_tv_channel)
        {
            if ($xml_tv_channel->getName() !== 'item')
            {
                hd_print("Error: unexpected node '" . $xml_tv_channel->getName() .
                    "'. Expected: 'item'");
                throw new Exception('Invalid XML document');
            }

            //hd_print(preg_replace('/\/res\//', "/resize/245x140/res/", strval($xml_tv_channel->img->url)));
            
            if (isset($active_channels[strval($xml_tv_channel->id)])) {
            	$channel =
                new I24hChannel(
                    strval($xml_tv_channel->id),
                    strval($xml_tv_channel->name),
                    preg_replace('/\/res\//', "/resize/245x140/res/", strval($xml_tv_channel->img->url)),
                    "/85.112.114.10-5091/".$this->session->get_sid_uid().'-%s-'.$xml_tv_channel->id, //strval($xml_tv_channel->streaming_url),
                    intval($xml_tv_channel->priorety),
                    14,  //past epg days
                    7,   //future epg days
                    "/85.112.114.12-7040/".$this->session->get_sid_uid().'-%s-'.$xml_tv_channel->id.'-%s', //strval($xml_tv_channel->archive_url),
                    false //intval($xml_tv_channel->is_protected) ? true : false
                    );
                $active_list[] = $channel;
            } else {
            	$channel =
                new I24hChannel(
                    strval($xml_tv_channel->id),
                    strval($xml_tv_channel->name),
                    preg_replace(array('/\/res\//','/\/([^\/]+)$/'), array("/resize/245x140/res/",'/lock_$1'), strval($xml_tv_channel->img->url)),
                    'http://ts://85.112.114.12:18987/238.254.0.2-1234|||dune_params|||ll_prebuf_for_http_ts:1,lower_cpu_load:1,psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0',
                    intval($xml_tv_channel->priorety),
                    14,  //past epg days
                    7,   //future epg days
                    'http://ts://85.112.114.12:18987/238.254.0.2-1234|||dune_params|||ll_prebuf_for_http_ts:1,lower_cpu_load:1,psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0', //strval($xml_tv_channel->archive_url),
                    false //intval($xml_tv_channel->is_protected) ? true : false
                    );
                $passive_list[] = $channel;
            }

            //$this->channels->put($channel);

            foreach ($xml_tv_channel->lnkCnl2Ctg->children() as $xml_tv_cat_id)
            {
                if ($xml_tv_cat_id->getName() !== 'item')
                {
                    hd_print("Error: unexpected lnkCnl2Ctg node '" . $xml_tv_cat_id->getName() .
                        "'. Expected: 'item'");
                    throw new Exception('Invalid XML document');
                }

                $tv_category_id = intval($xml_tv_cat_id->cat_id);
                //hd_print("$xml_tv_channel->id -> $tv_category_id");
                
                $group = $this->groups->get($tv_category_id);

                // Link group and channel.

                if (isset($group)) {
                    $channel->add_group($group);
                }
                //$group->add_channel($channel);
            }
        }
        
        foreach ($active_list as $channel) {
            $this->channels->put($channel);
            foreach ($channel->get_groups() as $group) {
            	if (isset($group)) {
            	    $group->add_channel($channel);
            	}
            }
        }

        foreach ($passive_list as $channel) {
            $this->channels->put($channel);
            foreach ($channel->get_groups() as $group) {
            	if (isset($group)) {
            	    $group->add_channel($channel);
            	}
            }
        }
        
        if (I24hConfig::CHANNEL_SORT_FUNC_CB != null)
            $this->channels->usort(I24hConfig::CHANNEL_SORT_FUNC_CB);
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies)
    {
        $this->session->ensure_logged_in($plugin_cookies);

        return parent::get_tv_info($media_url, &$plugin_cookies);
    }

    public function get_tv_stream_url($playback_url, &$plugin_cookies)
    {
        $url = $this->session->api_get_stream_url($playback_url);
        if (substr($url, 0, 1) === '/') {
        	$ss_bin_path = DuneSystem::$properties['install_dir_path'] . '/bin';
        	$cmd = $ss_bin_path . "/play.sh $ss_bin_path > /tmp/rrr.log";
        	$s = shell_exec($cmd);
        	if (!empty($s))
        		hd_print("$cmd - $s");
        	return 'http://ts://localhost:8081'.$url.'|||dune_params|||ll_prebuf_for_http_ts:1,lower_cpu_load:1,psi_opt:2,buffering_ms:300,ss_for_http_ts:1';
        } else {
        	return $url;
        }
    }

    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
    {

        $this->ensure_channels_loaded($plugin_cookies);
                                                                                                                                                                 
        echo "get url: $channel_id, $archive_ts\n";

        if ($archive_ts > 0) {
                $url = sprintf($this->get_channel($channel_id)->get_archive_url(), $this->session->get_sid(), $archive_ts);
                if (substr($url, 0, 1) === '/') {
                        $ss_bin_path = DuneSystem::$properties['install_dir_path'] . '/bin';
                        $cmd = $ss_bin_path . "/play.sh $ss_bin_path > /tmp/rrr.log";
                        $s = shell_exec($cmd);
                        if (!empty($s))
                                hd_print("$cmd - $s");
                            return 'http://ts://localhost:8081'.$url.'|||dune_params|||ll_prebuf_for_http_ts:1,lower_cpu_load:1,psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
                } else {
                        return $url."|||dune_params|||,ss_for_http_ts:1";
                }
        } else {
                $url = sprintf($this->get_channel($channel_id)->get_streaming_url(), $this->session->get_sid());
                if (substr($url, 0, 1) === '/') {
                        $ss_bin_path = DuneSystem::$properties['install_dir_path'] . '/bin';
                        $cmd = $ss_bin_path . "/play.sh $ss_bin_path > /tmp/rrr.log";
                        $s = shell_exec($cmd);
                        if (!empty($s))
                                hd_print("$cmd - $s");
                            return 'http://ts://localhost:8081'.$url.'|||dune_params|||ll_prebuf_for_http_ts:1,lower_cpu_load:1,psi_opt:2,buffering_ms:300,ss_for_http_ts:1,keep_pic:0';
                } else {
                        return $url."|||dune_params|||,ss_for_http_ts:1";
                }
        }                                                                                                                                                                
        return;                                                                                                                                                          

        /*

        $this->ensure_channels_loaded($plugin_cookies);

        echo "get url: $channel_id, $archive_ts\n";

        $url = sprintf(I24H_GET_URL_URL, I24H::$SERVER,
        		$plugin_cookies->user_name,
        		$plugin_cookies->password
        	);
        
        $url .= "&cid=$channel_id";

        if (intval($archive_ts) > 0)
            $url .= "&start_ts=$archive_ts";
        if (isset($protect_code) && $protect_code !== '')
            $url .= "&protect_code=$protect_code";

        return $url;*/
    }
    
    ///////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
    {
        if (I24hConfig::EPG_URL_FORMAT == null)
            return array();

        try
        {
            $doc =
                HD::http_get_document(
                    sprintf(
                        I24hConfig::EPG_URL_FORMAT,
                        $this->session->get_token(),
                        $channel_id,
                        $day_start_ts,
                        $day_start_ts+24*3600));
        }
        catch (Exception $e)
        {
            throw new DuneException(
                'HTTP network error', 0,
                ActionFactory::show_error(
                    true,
                    'Ошибка связи',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте подключение к сети Интернет.')));
        }
     
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print( "XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'response')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'NewDataSet'");
            throw new Exception('Invalid XML document');
        }

        $epg = array();

        foreach ($xml->children() as $epg_item)
        {
            if ($epg_item->getName() !== 'item')
            {
                hd_print("Error: unexpected node '" . $epg_item->getName() .
                    "'. Expected: 'item'");
                throw new Exception('Invalid XML document');
            }

            $epg[] =
                new DefaultEpgItem(
                    strval($epg_item->name),
                    strval($epg_item->txt),
                    intval($epg_item->tm),
                    intval($epg_item->etm));
        }

        return
            new EpgIterator(
                $epg,
                $day_start_ts,
                $day_start_ts + 86400);
    }

    public function folder_entered(MediaURL $media_url, &$plugin_cookies)
    {
        if (!isset($media_url->screen_id) ||
            $media_url->screen_id === TvGroupListScreen::ID)
        {
           // $this->session->logout();
        }

        $this->session->ensure_logged_in($plugin_cookies);
    }
    
    public function have_channel($channel_id)
    {
        $c = $this->channels->get($channel_id);

        if (is_null($c))
            return false;

        return true;
    }

}

///////////////////////////////////////////////////////////////////////////
?>
